# -*- coding: utf-8 -*-
"""
@author: Tihomir Heidelberg
"""

import os

WEB_BOILER_WEBROOT = 'https://www.web-boiler.com'

WEB_BOILER_STOMP_LOGIN_USERNAME = "appuser"
WEB_BOILER_STOMP_LOGIN_PASSCODE = "appuser"
WEB_BOILER_STOMP_URL = 'wss://web-boiler.com:15671/ws'
WEB_BOILER_STOMP_DEVICE_TOPIC = "/topic/cm.inst."
WEB_BOILER_STOMP_NOTIFICATION_TOPIC = "/queue/notification"
